String actorsFile = "actors.csv";
String eventsFile = "hotness.csv";
Actor[] actors;
Table CSV;
int n;
float eps = 0.1;
float oldTimestamp;
int counter;
int diameter = 20;
int expand = 10;

void setup() {
  size(1000, 280);
  background(255);

  // load actor data from CSV
  loadActors(actorsFile);

  // load event data from CSV
  CSV = loadTable(eventsFile, "header");

  // number of events
  n = CSV.getRowCount();

  // timestamp of first event 
  oldTimestamp = CSV.getRow(0).getFloat("timestamp");

  // event counter
  counter = 0;
}

void draw() {

  background(255);
  // read one event 
  TableRow row = CSV.getRow(counter);
  int to = row.getInt("to");
  float toReward = row.getFloat("toReward");
  float timestamp = row.getFloat("timestamp");
  
  // increase rating for 'to' actor. 
  // Notice: indexes of vector actors starts from 0 but "to" values starts from 1
  actors[to-1].update(toReward);
  
  // decrease rating of all actors proportionally to the elapsed time from previous event
  float deflate = -eps * (timestamp - oldTimestamp) / (60 * 60 * 24);
  for (int j = 0; j < actors.length; j++) {
    actors[j].update(deflate);
  }

   // show and label all actors
  for (int j = 0; j < actors.length; j++) {
    actors[j].show();
    actors[j].label();
  }

  // update previous timestamp
  oldTimestamp = timestamp;

  // increase event counter
  counter += 1;

  // stop when all events have been processed
  if (counter == n) {
    noLoop();
  }
}

// read actor data from CSV and create actor vector
void loadActors(String filename) {
  Table CSV = loadTable(filename, "header");
  int n = CSV.getRowCount();
  actors = new Actor[n];

  // build actor vector
  for (int i = 0; i < n; i++) {
    TableRow row = CSV.getRow(i);
    int id = row.getInt("id");
    String hash = row.getString("hash");
    actors[i] = new Actor(id, hash, i * (width / n), height);
  }
}


// each user is an actor
class Actor {

  int id; // id
  String hash; // hash
  float x, y; // position 

  Actor(int _id, String _hash, float _x, float _y) {
    id = _id;
    hash = _hash;
    x = _x;
    y = _y;
  }

  // show the actor as a bubble
  void show() { 
    if (y < height) {
      noStroke(); 
      float alpha = map(y, height, 0, 50, 255);
      fill(255, 0, 0, alpha);
      ellipse(x, y, diameter, diameter);
    }
  }

  // label the actor's bubble with its id
  void label() { 
    if (y < height) {
      fill(0);
      textSize(10);
      text(id, x - 5, y - 10);
    }
  }

  
  // update y-position of the actor
  void update(float rw) {
    y -= expand * rw;
    if (y > height)  y = height; 
    if (y < 0)       y = 0; // do not go below 0
  }
}
